<?php

require_once( SCLEGN_PATH . '/vendor/autoload.php' );
use Abraham\TwitterOAuth\TwitterOAuth;
class Meow_SCLEGN_Services_Twitter
{
  public function __construct() {
    add_action( 'plugins_loaded', array( $this, 'plugins_loaded' ), 10 );
  }

  public function plugins_loaded() {

    add_filter( 'sclegn_modules', function( $modules ) {
      if ( !in_array( 'twitter', $modules ) ) {
        $modules[] = [ 
          'type' => 'twitter', 
          'instance' => $this
        ];
      }
      return $modules;
    }, 10, 1 );

    add_filter( 'sclegn_accounts', function( $accounts ) {
      $services = get_option( 'sclegn_services', array() );
      foreach ( $services as &$service ) {
        if ( $service['type'] === 'twitter' ) {
          $accounts[] = [ 
            'type' => 'twitter',
            'name' => $service['name'],
            'serviceId' => $service['id'],
            'accountId' => $service['id'],
            'instance' => $this
          ];
        }
      }
      return $accounts;
    }, 10, 1 );
  }

  private function create_connection( $settings ) {

    // App Key === API Key === Consumer API Key === Consumer Key === Customer Key === oauth_consumer_key
    // App Key Secret === API Secret Key === Consumer Secret === Consumer Key === Customer Key === oauth_consumer_secret

    if ( empty( $settings['api_key'] ) ) {
      throw new Exception("The API Key is missing.");
    }
    if ( empty( $settings['api_secret_key'] ) ) {
      throw new Exception("The API Secret Key is missing.");
    }
    if ( empty( $settings['access_token'] ) ) {
      throw new Exception("The access token is missing.");
    }
    if ( empty( $settings['access_token_secret'] ) ) {
      throw new Exception("The access token secret is missing.");
    }
    $connection = new TwitterOAuth( $settings['api_key'], $settings['api_secret_key'], 
      $settings['access_token'], $settings['access_token_secret'] );
    return $connection;
  }

  public function post( $social_post, $settings ) {
    $media_ids = [];
    if ( isset( $social_post['thumbnail_paths'] ) && count( $social_post['thumbnail_paths'] ) > 0 ) {
      foreach ( $social_post['thumbnail_paths'] as $thumbnail_path ) {
        $media = $this->upload( $thumbnail_path, $settings );
        $media_ids[] = $media->media_id_string;
      }
    }
    $parameters = [
        'status' => $social_post['post_content'],
        'media_ids' => implode( ',', $media_ids )
    ];
    $connection = $this->create_connection( $settings );
    $statues = $connection->post( "statuses/update", $parameters );
    if ( $connection->getLastHttpCode() !== 200 ) {
      throw new Exception( $statues->errors[0]->message, $statues->errors[0]->code );
    }
    return [
      'posted_id' => $statues->id_str,
      'post_date' => get_date_from_gmt( $statues->created_at, 'Y-m-d H:i:s' ),
    ];
  }

  public function upload( $media, $settings ) {
    $connection = $this->create_connection( $settings );
    $media = $connection->upload( 'media/upload', [ 'media' => $media ] );
    if ( $connection->getLastHttpCode() !== 200 ) {
      throw new Exception( $media->errors[0]->message, $media->errors[0]->code );
    }
    return $media;
  }

  public function request_service( $settings, $action, $params ) {
    if ( $action === "test" ) {
      $connection = $this->create_connection( $settings );
      $content = $connection->get("account/verify_credentials");
      if ( $content && !empty( $content->errors ) ) {
        throw new Exception( $content->errors[0]->message, $content->errors[0]->code );
      }
      if ( $content && !empty( $content->name ) ) {
        return $content->name;
      }
    }
    return false;
  }

  public function stats( $posted_id, $settings, $account_id ) {
    $connection = $this->create_connection( $settings );
    $content = $connection->get('statuses/show/' . $posted_id );
    if ( $connection->getLastHttpCode() !== 200 ) {
      throw new Exception( $content->errors[0]->message, $content->errors[0]->code );
    }
    return $content;
  }
}